@@ -1,4 +1,4 @@
-/* $Id: os_core_win32.c 3553 2011-05-05 06:14:19Z nanang $ */
+/* $Id: os_core_win32.c 3986 2012-03-22 11:29:20Z nanang $ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -116,6 +116,11 @@ struct pj_atomic_t
     long value;
 };
 
+/*
+ * Flag and reference counter for PJLIB instance.
+ */
+static int initialized;
+
 /*
  * Static global variables.
  */
@@ -142,6 +147,12 @@ PJ_DEF(pj_status_t) pj_init(void)
     pj_str_t guid;
     pj_status_t rc;
 
+    /* Check if PJLIB have been initialized */
+    if (initialized) {
+	++initialized;
+	return PJ_SUCCESS;
+    }
+
     /* Init Winsock.. */
     if (WSAStartup(MAKEWORD(2,0), &wsa) != 0) {
 	return PJ_RETURN_OS_ERROR(WSAGetLastError());
@@ -187,6 +198,10 @@ PJ_DEF(pj_status_t) pj_init(void)
     }
 #endif   
 
+    /* Flag PJLIB as initialized */
+    ++initialized;
+    pj_assert(initialized == 1);
+
     PJ_LOG(4,(THIS_FILE, "pjlib %s for win32 initialized",
 	      PJ_VERSION));
 
@@ -213,6 +228,11 @@ PJ_DEF(void) pj_shutdown()
 {
     int i;
 
+    /* Only perform shutdown operation when 'initialized' reaches zero */
+    pj_assert(initialized > 0);
+    if (--initialized != 0)
+	return;
+
     /* Display stack usage */
 #if defined(PJ_OS_HAS_CHECK_STACK) && PJ_OS_HAS_CHECK_STACK!=0
     {
