@@ -2,6 +2,7 @@
  *  Copyright (C) 2004, 2005, 2006, 2008, 2009, 2010, 2011 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
+ *  Author: Андрей Лухнов <aol.nnov@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,7 +54,7 @@ class AlsaLayer : public AudioLayer {
         /**
          * Constructor
          */
-        AlsaLayer();
+        AlsaLayer(const AudioPreference &pref);
 
         /**
          * Destructor
@@ -84,18 +85,10 @@ class AlsaLayer : public AudioLayer {
 
         /**
          * Scan the sound card available on the system
-         * @param stream To indicate whether we are looking for capture devices or playback devices
-         *		   SFL_PCM_CAPTURE
-         *		   SFL_PCM_PLAYBACK
-         *		   SFL_PCM_BOTH
          * @return std::vector<std::string> The vector containing the string description of the card
          */
-        virtual std::vector<std::string> getAudioDeviceList(AudioStreamDirection dir) const;
-
-        /**
-         * Returns a map of audio device hardware description and index
-         */
-        std::vector<HwIDPair> getAudioDeviceIndexMap(AudioStreamDirection dir) const;
+        virtual std::vector<std::string> getCaptureDeviceList() const;
+        virtual std::vector<std::string> getPlaybackDeviceList() const;
 
         /**
          * Check if the given index corresponds to an existing sound card and supports the specified streaming mode
@@ -107,7 +100,7 @@ class AlsaLayer : public AudioLayer {
          * @return bool True if it exists and supports the mode
          *		    false otherwise
          */
-        static bool soundCardIndexExists(int card, int stream);
+        static bool soundCardIndexExists(int card, PCMType stream);
 
         /**
          * An index is associated with its string description
@@ -115,7 +108,8 @@ class AlsaLayer : public AudioLayer {
          * @return	int	  Its index
          */
         int getAudioDeviceIndex(const std::string &description) const;
-        
+        std::string getAudioDeviceName(int index, PCMType type) const;
+
         void playback(int maxSamples);
         void capture();
 
@@ -126,7 +120,7 @@ class AlsaLayer : public AudioLayer {
          * @return int The index of the card used for capture
          *                     0 for the first available card on the system, 1 ...
          */
-        int getIndexCapture() const {
+        virtual int getIndexCapture() const {
             return indexIn_;
         }
 
@@ -135,7 +129,7 @@ class AlsaLayer : public AudioLayer {
          * @return int The index of the card used for playback
          *                     0 for the first available card on the system, 1 ...
          */
-        int getIndexPlayback() const {
+        virtual int getIndexPlayback() const {
             return indexOut_;
         }
 
@@ -144,12 +138,16 @@ class AlsaLayer : public AudioLayer {
          * @return int The index of the card used for ringtone
          *                 0 for the first available card on the system, 1 ...
          */
-        int getIndexRingtone() const {
+        virtual int getIndexRingtone() const {
             return indexRing_;
         }
 
     private:
         friend class AlsaThread;
+        /**
+         * Returns a map of audio device hardware description and index
+         */
+        std::vector<HwIDPair> getAudioDeviceIndexMap(bool getCapture) const;
 
         /**
          * Calls snd_pcm_open and retries if device is busy, since dmix plugin
@@ -208,6 +206,8 @@ class AlsaLayer : public AudioLayer {
          */
         int read(void* buffer, int toCopy);
 
+        virtual void updatePreference(AudioPreference &pref, int index, PCMType type);
+
         /**
          * Handles to manipulate playback stream
          * ALSA Library API
