@@ -54,17 +54,30 @@ namespace ost {
 class Time;
 }
 
-enum AudioStreamDirection { AUDIO_STREAM_CAPTURE, AUDIO_STREAM_PLAYBACK };
-
 class AudioLayer {
+
     private:
         NON_COPYABLE(AudioLayer);
 
     public:
+        enum PCMType {
+            SFL_PCM_BOTH = 0x0021,          /** To open both playback and capture devices */
+            SFL_PCM_PLAYBACK = 0x0022,      /** To open playback device only */
+            SFL_PCM_CAPTURE = 0x0023,       /** To open capture device only */
+            SFL_PCM_RINGTONE = 0x0024       /** To open the ringtone device only */
+        };
+
         AudioLayer();
         virtual ~AudioLayer() {}
 
-        virtual std::vector<std::string> getAudioDeviceList(AudioStreamDirection dir) const = 0;
+        virtual std::vector<std::string> getCaptureDeviceList() const = 0;
+        virtual std::vector<std::string> getPlaybackDeviceList() const = 0;
+
+        virtual int getAudioDeviceIndex(const std::string& name) const = 0;
+        virtual std::string getAudioDeviceName(int index, PCMType type) const = 0;
+        virtual int getIndexCapture() const = 0;
+        virtual int getIndexPlayback() const = 0;
+        virtual int getIndexRingtone() const = 0;
 
         /**
          * Start the capture stream and prepare the playback stream.
@@ -123,7 +136,7 @@ class AudioLayer {
         static double ampluitudeDBToLinear(double value) {
             return pow(10.0, value / 20.0);
         }
- 
+
         /**
          * Set capture stream gain (microphone)
          */
@@ -132,7 +145,7 @@ class AudioLayer {
         }
 
         /**
-         * Set capture stream gain (microphone) 
+         * Set capture stream gain (microphone)
          */
        unsigned int getCaptureGain(void) {
             return captureGain_;
@@ -146,7 +159,7 @@ class AudioLayer {
         }
 
         /**
-         * Get playback stream gain (speaker) 
+         * Get playback stream gain (speaker)
          */
         unsigned int getPlaybackGain(void) {
             return playbackGain_;
@@ -168,7 +181,7 @@ class AudioLayer {
             return &mutex_;
         }
 
-	/**
+        /**
          * Emit an audio notification on incoming calls
          */
         void notifyIncomingCall();
@@ -183,10 +196,11 @@ class AudioLayer {
          */
         static unsigned int playbackGain_;
 
-    protected:
+        virtual void updatePreference(AudioPreference &pref, int index, PCMType type) = 0;
 
+    protected:
         /**
-         * Wether or not the audio layer stream is started
+         * Whether or not the audio layer stream is started
          */
         bool isStarted_;
 
@@ -211,11 +225,6 @@ class AudioLayer {
          */
         DcBlocker dcblocker_;
 
-        /**
-         * Configuration file for this 
-         */
-        AudioPreference &audioPref;
-
         /**
          * Manage sampling rate conversion
          */
