@@ -48,18 +48,9 @@ class SIPCall;
 
 namespace sfl {
 
-// Frequency (in packet number)
-#define RTP_TIMESTAMP_RESET_FREQ 100
-
-static const int schedulingTimeout = 4000;
-static const int expireTimeout = 1000000;
-
 // G.722 VoIP is typically carried in RTP payload type 9.[2] Note that IANA records the clock rate for type 9 G.722 as 8 kHz
 // (instead of 16 kHz), RFC3551[3]  clarifies that this is due to a historical error and is retained in order to maintain backward
 // compatibility. Consequently correct implementations represent the value 8,000 where required but encode and decode audio at 16 kHz.
-static const int g722PayloadType = 9;
-static const int g722RtpClockRate = 8000;
-static const int g722RtpTimeincrement = 160;
 
 inline uint32
 timeval2microtimeout(const timeval& t)
@@ -67,6 +58,13 @@ timeval2microtimeout(const timeval& t)
     return ((t.tv_sec * 1000000ul) + t.tv_usec);
 }
 
+struct DTMFEvent {
+    DTMFEvent(int digit);
+    ost::RTPPacket::RFC2833Payload payload;
+    bool newevent;
+    int length;
+};
+
 /**
  * Class meant to store internal data in order to encode/decode,
  * resample, process, and packetize audio streams. This class should not be
@@ -76,7 +74,11 @@ class AudioRtpRecord {
     public:
         AudioRtpRecord();
         ~AudioRtpRecord();
+        std::string callId_;
+        int codecSampleRate_;
+        std::list<DTMFEvent> dtmfQueue_;
 
+    private:
         AudioCodec *audioCodec_;
         ost::Mutex audioCodecMutex_;
         int codecPayloadType_;
@@ -87,24 +89,30 @@ class AudioRtpRecord {
         std::tr1::array<unsigned char, DEC_BUFFER_SIZE> encodedData_;
         SamplerateConverter *converterEncode_;
         SamplerateConverter *converterDecode_;
-        int codecSampleRate_;
         int codecFrameSize_;
         int converterSamplingRate_;
-        std::list<int> dtmfQueue_;
-        SFLDataFormat fadeFactor_;
+        double fadeFactor_;
+
+#if HAVE_SPEEXDSP
         NoiseSuppress *noiseSuppressEncode_;
         NoiseSuppress *noiseSuppressDecode_;
         ost::Mutex audioProcessMutex_;
-        std::string callId_;
+#endif
+
         unsigned int dtmfPayloadType_;
 
-    private:
+        bool isDead();
         friend class AudioRtpRecordHandler;
         /**
         * Ramp In audio data to avoid audio click from peer
         */
         void fadeInDecodedData(size_t size);
         NON_COPYABLE(AudioRtpRecord);
+#ifdef CCPP_PREFIX
+        ost::AtomicCounter dead_;
+#else
+        ucommon::atomic::counter dead_;
+#endif
 };
 
 
@@ -138,7 +146,7 @@ class AudioRtpRecordHandler {
             return audioRtpRecord_.hasDynamicPayloadType_;
         }
 
-        int DtmfPending() const {
+        bool hasDTMFPending() const {
             return not audioRtpRecord_.dtmfQueue_.empty();
         }
 
@@ -148,7 +156,9 @@ class AudioRtpRecordHandler {
 
         void initBuffers();
 
+#if HAVE_SPEEXDSP
         void initNoiseSuppress();
+#endif
 
         /**
          * Encode audio data from mainbuffer
@@ -174,9 +184,7 @@ class AudioRtpRecordHandler {
         AudioRtpRecord audioRtpRecord_;
 
     private:
-
         const std::string id_;
-        EchoSuppress echoCanceller;
         GainControl gainController;
 };
 }
