@@ -68,67 +68,45 @@ void AudioRtpSession::updateSessionMedia(AudioCodec &audioCodec)
 
     Manager::instance().audioSamplingRateChanged(audioRtpRecord_.codecSampleRate_);
 
+#if HAVE_SPEEXDSP
     if (lastSamplingRate != audioRtpRecord_.codecSampleRate_) {
         DEBUG("Update noise suppressor with sampling rate %d and frame size %d",
               getCodecSampleRate(), getCodecFrameSize());
         initNoiseSuppress();
     }
+#endif
 }
 
 void AudioRtpSession::setSessionMedia(AudioCodec &audioCodec)
 {
     setRtpMedia(&audioCodec);
 
-    // store codec info locally
-    int payloadType = getCodecPayloadType();
-    int frameSize = getCodecFrameSize();
-    int smplRate = getCodecSampleRate();
-    bool dynamic = getHasDynamicPayload();
-
     // G722 requires timestamp to be incremented at 8kHz
-    if (payloadType == g722PayloadType)
-        timestampIncrement_ = g722RtpTimeincrement;
-    else
-        timestampIncrement_ = frameSize;
+    const ost::PayloadType payloadType = getCodecPayloadType();
+    if (payloadType == ost::sptG722) {
+        const int G722_RTP_TIME_INCREMENT = 160;
+        timestampIncrement_ = G722_RTP_TIME_INCREMENT;
+    } else
+        timestampIncrement_ = getCodecFrameSize();
 
-    DEBUG("Codec payload: %d", payloadType);
-    DEBUG("Codec sampling rate: %d", smplRate);
-    DEBUG("Codec frame size: %d", frameSize);
-    DEBUG("RTP timestamp increment: %d", timestampIncrement_);
-
-    if (payloadType == g722PayloadType) {
-        DEBUG("Setting G722 payload format");
-        queue_.setPayloadFormat(ost::DynamicPayloadFormat((ost::PayloadType) payloadType, g722RtpClockRate));
+    if (payloadType == ost::sptG722) {
+        const int G722_RTP_CLOCK_RATE = 8000;
+        queue_.setPayloadFormat(ost::DynamicPayloadFormat( payloadType, G722_RTP_CLOCK_RATE));
     } else {
-        if (dynamic) {
-            DEBUG("Setting dynamic payload format");
-            queue_.setPayloadFormat(ost::DynamicPayloadFormat((ost::PayloadType) payloadType, smplRate));
-        } else {
-            DEBUG("Setting static payload format");
-            queue_.setPayloadFormat(ost::StaticPayloadFormat((ost::StaticPayloadType) payloadType));
-        }
+        if (getHasDynamicPayload())
+            queue_.setPayloadFormat(ost::DynamicPayloadFormat(payloadType, getCodecSampleRate()));
+        else
+            queue_.setPayloadFormat(ost::StaticPayloadFormat(static_cast<ost::StaticPayloadType>(payloadType)));
     }
 }
 
-void AudioRtpSession::incrementTimestampForDTMF()
-{
-    timestamp_ += timestampIncrement_;
-}
-
 void AudioRtpSession::sendDtmfEvent()
 {
-    ost::RTPPacket::RFC2833Payload payload;
+    DTMFEvent &dtmf(audioRtpRecord_.dtmfQueue_.front());
+    DEBUG("Send RTP Dtmf (%d)", dtmf.payload.event);
 
-    payload.event = audioRtpRecord_.dtmfQueue_.front();
-    payload.ebit = false; // end of event bit
-    payload.rbit = false; // reserved bit
-    payload.duration = 1; // duration for this event
-
-    audioRtpRecord_.dtmfQueue_.pop_front();
-
-    DEBUG("Send RTP Dtmf (%d)", payload.event);
-
-    incrementTimestampForDTMF();
+    const int increment = getIncrementForDTMF();
+    timestamp_ += increment;
 
     // discard equivalent size of audio
     processDataEncode();
@@ -136,13 +114,29 @@ void AudioRtpSession::sendDtmfEvent()
     // change Payload type for DTMF payload
     queue_.setPayloadFormat(ost::DynamicPayloadFormat((ost::PayloadType) getDtmfPayloadType(), 8000));
 
-    queue_.setMark(true);
-    queue_.sendImmediate(timestamp_, (const unsigned char *)(&payload), sizeof(payload));
-    queue_.setMark(false);
+    // Set marker in case this is a new Event
+    if (dtmf.newevent)
+        queue_.setMark(true);
+    queue_.sendImmediate(timestamp_, (const unsigned char *) (& (dtmf.payload)), sizeof (ost::RTPPacket::RFC2833Payload));
 
-    // get back the payload to audio
+    // This is no longer a new event
+    if (dtmf.newevent) {
+        dtmf.newevent = false;
+        queue_.setMark(false);
+    }
+
+    // restore the payload to audio
     const ost::StaticPayloadFormat pf(static_cast<ost::StaticPayloadType>(getCodecPayloadType()));
     queue_.setPayloadFormat(pf);
+
+    // decrease length remaining to process for this event
+    dtmf.length -= increment;
+    dtmf.payload.duration++;
+    // next packet is going to be the last one
+    if ((dtmf.length - increment) < increment)
+        dtmf.payload.ebit = true;
+    if (dtmf.length < increment)
+        audioRtpRecord_.dtmfQueue_.pop_front();
 }
 
 
@@ -182,11 +176,13 @@ void AudioRtpSession::sendMicData()
 
 void AudioRtpSession::setSessionTimeouts()
 {
+    const int schedulingTimeout = 4000;
+    const int expireTimeout = 1000000;
     DEBUG("Set session scheduling timeout (%d) and expireTimeout (%d)",
-            sfl::schedulingTimeout, sfl::expireTimeout);
+          schedulingTimeout, expireTimeout);
 
-    queue_.setSchedulingTimeout(sfl::schedulingTimeout);
-    queue_.setExpireTimeout(sfl::expireTimeout);
+    queue_.setSchedulingTimeout(schedulingTimeout);
+    queue_.setExpireTimeout(expireTimeout);
 }
 
 void AudioRtpSession::setDestinationIpAddress()
@@ -239,7 +235,9 @@ int AudioRtpSession::startRtpThread(AudioCodec &audiocodec)
     setSessionTimeouts();
     setSessionMedia(audiocodec);
     initBuffers();
+#if HAVE_SPEEXDSP
     initNoiseSuppress();
+#endif
 
     queue_.enableStack();
     thread_.start();
@@ -253,4 +251,9 @@ bool AudioRtpSession::onRTPPacketRecv(ost::IncomingRTPPkt&)
     return true;
 }
 
+int AudioRtpSession::getIncrementForDTMF() const
+{
+    return timestampIncrement_;
+}
+
 }
