@@ -66,6 +66,8 @@ class AudioRtpSession : public AudioRtpRecordHandler {
          */
         void updateDestinationIpAddress();
 
+        virtual int getIncrementForDTMF() const;
+
     protected:
         /**
          * Set the audio codec for this RTP session
@@ -122,11 +124,6 @@ class AudioRtpSession : public AudioRtpRecordHandler {
          */
         void receiveSpeakerData();
 
-        /**
-         * Increment timestamp for DTMF event
-         */
-        virtual void incrementTimestampForDTMF();
-
         // Main destination address for this rtp session.
         // Stored in case or reINVITE, which may require to forget
         // this destination and update a new one.
