@@ -66,11 +66,9 @@ void AudioSymmetricRtpSession::AudioRtpThread::run()
 
     TimerPort::setTimer(threadSleep);
 
-    DEBUG("Entering Audio rtp thread main loop");
-
     while (running_) {
         // Send session
-        if (rtpSession_.DtmfPending())
+        if (rtpSession_.hasDTMFPending())
             rtpSession_.sendDtmfEvent();
         else
             rtpSession_.sendMicData();
@@ -79,8 +77,6 @@ void AudioSymmetricRtpSession::AudioRtpThread::run()
 
         TimerPort::incTimer(threadSleep);
     }
-
-    DEBUG("Leaving audio rtp thread loop");
 }
 
 void AudioSymmetricRtpSession::setSessionMedia(AudioCodec &audioCodec)
