@@ -37,13 +37,15 @@
 
 #include "audiocodecfactory.h"
 #include <cstdlib>
+#include <dlfcn.h>
 #include <algorithm> // for std::find
 #include <dlfcn.h>
 #include "fileutils.h"
+#include "array_size.h"
 #include "logger.h"
 
 AudioCodecFactory::AudioCodecFactory() :
-    codecsMap_(), defaultCodecOrder_(), libCache_(), codecInMemory_()
+    codecsMap_(), defaultCodecList_(), libCache_(), codecInMemory_()
 {
     typedef std::vector<sfl::Codec*> CodecVector;
     CodecVector codecDynamicList(scanCodecDirectory());
@@ -61,11 +63,9 @@ AudioCodecFactory::AudioCodecFactory() :
 
 void AudioCodecFactory::setDefaultOrder()
 {
-    defaultCodecOrder_.clear();
-    CodecsMap::const_iterator iter;
-
-    for (iter = codecsMap_.begin(); iter != codecsMap_.end(); ++iter)
-        defaultCodecOrder_.push_back(iter->first);
+    defaultCodecList_.clear();
+    for (CodecsMap::const_iterator i = codecsMap_.begin(); i != codecsMap_.end(); ++i)
+        defaultCodecList_.push_back(i->first);
 }
 
 std::string
@@ -79,7 +79,7 @@ AudioCodecFactory::getCodecName(int payload) const
         return "";
 }
 
-std::vector<int32_t >
+std::vector<int32_t>
 AudioCodecFactory::getAudioCodecList() const
 {
     std::vector<int32_t> list;
@@ -126,15 +126,15 @@ int AudioCodecFactory::getSampleRate(int payload) const
 
 void AudioCodecFactory::saveActiveCodecs(const std::vector<std::string>& list)
 {
-    defaultCodecOrder_.clear();
+    defaultCodecList_.clear();
     // list contains the ordered payload of active codecs picked by the user
-    // we used the CodecOrder vector to save the order.
+    // we used the codec vector to save the order.
 
     for (std::vector<std::string>::const_iterator iter = list.begin(); iter != list.end(); ++iter) {
         int payload = std::atoi(iter->c_str());
 
         if (isCodecLoaded(payload))
-            defaultCodecOrder_.push_back((int) payload);
+            defaultCodecList_.push_back(static_cast<int>(payload));
     }
 }
 
@@ -198,7 +198,7 @@ std::vector<sfl::Codec*> AudioCodecFactory::scanCodecDirectory()
 
 sfl::Codec* AudioCodecFactory::loadCodec(const std::string &path)
 {
-    void * codecHandle = dlopen(path.c_str() , RTLD_LAZY);
+    void * codecHandle = dlopen(path.c_str(), RTLD_LAZY);
 
     if (!codecHandle) {
         ERROR("%s", dlerror());
@@ -207,7 +207,7 @@ sfl::Codec* AudioCodecFactory::loadCodec(const std::string &path)
 
     dlerror();
 
-    create_t* createCodec = (create_t*) dlsym(codecHandle , "create");
+    create_t* createCodec = (create_t*) dlsym(codecHandle, CODEC_ENTRY_SYMBOL);
     char *error = dlerror();
 
     if (error) {
@@ -245,7 +245,7 @@ sfl::Codec* AudioCodecFactory::instantiateCodec(int payload) const
 
     for (iter = codecInMemory_.begin(); iter != codecInMemory_.end(); ++iter) {
         if (iter->first->getPayloadType() == payload) {
-            create_t* createCodec = (create_t*) dlsym(iter->second , "create");
+            create_t* createCodec = (create_t*) dlsym(iter->second , CODEC_ENTRY_SYMBOL);
 
             char *error = dlerror();
 
@@ -274,36 +274,31 @@ bool AudioCodecFactory::seemsValid(const std::string &lib)
 
     // Second: check the extension of the file name.
     // If it is different than SFL_CODEC_VALID_EXTEN , not a SFL shared library
-    if (lib.substr(lib.length() - suffix.length() , lib.length()) != suffix)
-        return false;
-
-
-#ifndef HAVE_SPEEX_CODEC
-
-    if (lib.substr(prefix.length() , len) == "speex")
+    if (lib.substr(lib.length() - suffix.length(), lib.length()) != suffix)
         return false;
 
+    static const std::string validCodecs[] = {
+    "ulaw",
+    "alaw",
+    "g722",
+#ifdef HAVE_SPEEX_CODEC
+    "speex_nb",
+    "speex_wb",
+    "speex_ub",
 #endif
 
-#ifndef HAVE_GSM_CODEC
-
-    if (lib.substr(prefix.length() , len) == "gsm")
-        return false;
-
+#ifdef HAVE_GSM_CODEC
+    "gsm",
 #endif
 
-#ifndef BUILD_ILBC
-
-    if (lib.substr(prefix.length() , len) == "ilbc")
-        return false;
-
+#ifdef BUILD_ILBC
+    "ilbc",
 #endif
+    ""};
 
-    if (lib.substr(0, prefix.length()) == prefix)
-        if (lib.substr(lib.length() - suffix.length() , suffix.length()) == suffix)
-            return true;
-
-    return false;
+    const std::string name(lib.substr(prefix.length(), len));
+    const std::string *end = validCodecs + ARRAYSIZE(validCodecs);
+    return find(validCodecs, end, name) != end;
 }
 
 bool
@@ -323,7 +318,8 @@ bool AudioCodecFactory::isCodecLoaded(int payload) const
     return false;
 }
 
-std::vector <std::string> AudioCodecFactory::getCodecSpecifications(const int32_t& payload) const
+std::vector <std::string>
+AudioCodecFactory::getCodecSpecifications(const int32_t& payload) const
 {
     std::vector<std::string> v;
     std::stringstream ss;
