@@ -33,7 +33,12 @@
 #include "logger.h"
 #include <stdexcept>
 
-AudioStream::AudioStream(pa_context *c, pa_threaded_mainloop *m, const char *desc, int type, int smplrate, std::string& deviceName)
+AudioStream::AudioStream(pa_context *c,
+                         pa_threaded_mainloop *m,
+                         const char *desc,
+                         int type,
+                         unsigned samplrate,
+                         const std::string &deviceName)
     : audiostream_(0), mainloop_(m)
 {
     static const pa_channel_map channel_map = {
@@ -43,7 +48,7 @@ AudioStream::AudioStream(pa_context *c, pa_threaded_mainloop *m, const char *des
 
     pa_sample_spec sample_spec = {
         PA_SAMPLE_S16LE, // PA_SAMPLE_FLOAT32LE,
-        smplrate,
+        samplrate,
         1
     };
 
@@ -65,13 +70,21 @@ AudioStream::AudioStream(pa_context *c, pa_threaded_mainloop *m, const char *des
     attributes.minreq = (uint32_t) -1;
 
     pa_threaded_mainloop_lock(mainloop_);
+    const pa_stream_flags_t flags = static_cast<pa_stream_flags_t>(PA_STREAM_ADJUST_LATENCY |
+                                                                   PA_STREAM_AUTO_TIMING_UPDATE);
 
-    if (type == PLAYBACK_STREAM || type == RINGTONE_STREAM)
-        pa_stream_connect_playback(audiostream_, deviceName == "" ? NULL : deviceName.c_str(), &attributes,
-		(pa_stream_flags_t)(PA_STREAM_ADJUST_LATENCY|PA_STREAM_AUTO_TIMING_UPDATE), NULL, NULL);
-    else if (type == CAPTURE_STREAM)
-        pa_stream_connect_record(audiostream_, deviceName == "" ? NULL : deviceName.c_str(), &attributes,
-		(pa_stream_flags_t)(PA_STREAM_ADJUST_LATENCY|PA_STREAM_AUTO_TIMING_UPDATE));
+    if (type == PLAYBACK_STREAM || type == RINGTONE_STREAM) {
+        pa_stream_connect_playback(audiostream_,
+                                   deviceName.empty() ? NULL : deviceName.c_str(),
+                                   &attributes,
+                                   flags,
+                                   NULL, NULL);
+    } else if (type == CAPTURE_STREAM) {
+        pa_stream_connect_record(audiostream_,
+                                 deviceName.empty() ? NULL : deviceName.c_str(),
+                                 &attributes,
+                                 flags);
+    }
 
     pa_threaded_mainloop_unlock(mainloop_);
 
