@@ -2,6 +2,7 @@
  *  Copyright (C) 2004, 2005, 2006, 2008, 2009, 2010, 2011 Savoir-Faire Linux Inc.
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
+ *  Author: Андрей Лухнов <aol.nnov@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +39,7 @@
 #include "logger.h"
 #include "manager.h"
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <fstream>
 
 namespace {
@@ -70,7 +71,7 @@ std::ofstream outfileResampled ("testMicOuputResampled.raw", std::ifstream::bina
 std::ofstream outfile("testMicOuput.raw", std::ifstream::binary);
 #endif
 
-PulseLayer::PulseLayer()
+PulseLayer::PulseLayer(AudioPreference &pref)
     : playback_(0)
     , record_(0)
     , ringtone_(0)
@@ -80,10 +81,12 @@ PulseLayer::PulseLayer()
     , mic_buf_size_(0)
     , context_(0)
     , mainloop_(pa_threaded_mainloop_new())
+    , enumeratingSinks_(false)
+    , enumeratingSources_(false)
+    , preference_(pref)
 {
     setenv("PULSE_PROP_media.role", "phone", 1);
 
-
     if (!mainloop_)
         throw std::runtime_error("Couldn't create pulseaudio mainloop");
 
@@ -174,58 +177,96 @@ void PulseLayer::context_state_callback(pa_context* c, void *user_data)
 void PulseLayer::updateSinkList()
 {
     sinkList_.clear();
-    pa_context_get_sink_info_list(context_, sink_input_info_callback, this);
+    enumeratingSinks_ = true;
+    pa_operation *op = pa_context_get_sink_info_list(context_, sink_input_info_callback, this);
+    if (op != NULL)
+        pa_operation_unref(op);
 }
 
 void PulseLayer::updateSourceList()
 {
     sourceList_.clear();
-    pa_context_get_source_info_list(context_, source_input_info_callback, this);
+    enumeratingSources_ = true;
+    pa_operation *op = pa_context_get_source_info_list(context_, source_input_info_callback, this);
+    if (op != NULL)
+        pa_operation_unref(op);
 }
 
 bool PulseLayer::inSinkList(const std::string &deviceName) const
 {
-    return std::find(sinkList_.begin(), sinkList_.end(), deviceName) != sinkList_.end();
+    const bool found = std::find(sinkList_.begin(), sinkList_.end(), deviceName) != sinkList_.end();
+    DEBUG("seeking for %s in sinks. %s found", deviceName.c_str(), found ? "" : "NOT");
+    return found;
 }
 
 
 bool PulseLayer::inSourceList(const std::string &deviceName) const
 {
-    return std::find(sourceList_.begin(), sourceList_.end(), deviceName) != sourceList_.end();
+    const bool found = std::find(sourceList_.begin(), sourceList_.end(), deviceName) != sourceList_.end();
+    DEBUG("seeking for %s in sources. %s found", deviceName.c_str(), found ? "" : "NOT");
+    return found;
 }
 
-std::vector<std::string> PulseLayer::getAudioDeviceList(AudioStreamDirection dir) const
+std::vector<std::string> PulseLayer::getCaptureDeviceList() const
 {
-    if (AUDIO_STREAM_CAPTURE == dir)
-        return sinkList_;
-    else if (AUDIO_STREAM_PLAYBACK)
-        return sourceList_;
+    return sourceList_;
+}
+
+std::vector<std::string> PulseLayer::getPlaybackDeviceList() const
+{
+    return sinkList_;
+}
+
+int PulseLayer::getAudioDeviceIndex(const std::string& name) const
+{
+    int index = std::distance(sourceList_.begin(), std::find(sourceList_.begin(), sourceList_.end(), name));
+    if (index == std::distance(sourceList_.begin(), sourceList_.end())) {
+        // not found in sources, search in sinks then
+        index = std::distance(sinkList_.begin(), std::find(sinkList_.begin(), sinkList_.end(), name));
+    }
+    return index;
+}
+
+std::string PulseLayer::getAudioDeviceName(int index, PCMType type) const
+{
+    switch (type) {
+        case SFL_PCM_PLAYBACK:
+        case SFL_PCM_RINGTONE:
+            return sinkList_.at(index);
+        case SFL_PCM_CAPTURE:
+            return sourceList_.at(index);
+        default:
+            return "";
+    }
 }
 
 void PulseLayer::createStreams(pa_context* c)
 {
-    std::string playbackDevice(audioPref.getDevicePlayback());
-    std::string captureDevice(audioPref.getDeviceRecord());
-    std::string ringtoneDevice(audioPref.getDeviceRingtone());
+    while (enumeratingSinks_ or enumeratingSources_)
+        ost::Thread::sleep(20 /* ms */);
+
+    std::string playbackDevice(preference_.getPulseDevicePlayback());
+    std::string captureDevice(preference_.getPulseDeviceRecord());
+    std::string ringtoneDevice(preference_.getPulseDeviceRingtone());
     std::string defaultDevice = "";
 
     DEBUG("Devices:\n   playback: %s\n   record: %s\n   ringtone: %s",
            playbackDevice.c_str(), captureDevice.c_str(), ringtoneDevice.c_str());
 
     playback_ = new AudioStream(c, mainloop_, "SFLphone playback", PLAYBACK_STREAM, sampleRate_,
-                                inSourceList(playbackDevice) ? playbackDevice : defaultDevice);
+                                inSinkList(playbackDevice) ? playbackDevice : defaultDevice);
 
     pa_stream_set_write_callback(playback_->pulseStream(), playback_callback, this);
     pa_stream_set_moved_callback(playback_->pulseStream(), stream_moved_callback, this);
 
     record_ = new AudioStream(c, mainloop_, "SFLphone capture", CAPTURE_STREAM, sampleRate_,
-                              inSinkList(captureDevice) ? captureDevice : defaultDevice);
+                              inSourceList(captureDevice) ? captureDevice : defaultDevice);
 
     pa_stream_set_read_callback(record_->pulseStream() , capture_callback, this);
     pa_stream_set_moved_callback(record_->pulseStream(), stream_moved_callback, this);
 
     ringtone_ = new AudioStream(c, mainloop_, "SFLphone ringtone", RINGTONE_STREAM, sampleRate_,
-                                inSourceList(ringtoneDevice) ? ringtoneDevice : defaultDevice);
+                                inSinkList(ringtoneDevice) ? ringtoneDevice : defaultDevice);
 
     pa_stream_set_write_callback(ringtone_->pulseStream(), ringtone_callback, this);
     pa_stream_set_moved_callback(ringtone_->pulseStream(), stream_moved_callback, this);
@@ -236,44 +277,22 @@ void PulseLayer::createStreams(pa_context* c)
     flushUrgent();
 }
 
+namespace {
+    // Delete stream and zero out its pointer
+    void
+    cleanupStream(AudioStream *&stream)
+    {
+        delete stream;
+        stream = 0;
+    }
+}
+
 
 void PulseLayer::disconnectAudioStream()
 {
-    if (playback_) {
-        if (playback_->pulseStream()) {
-            const char *name = pa_stream_get_device_name(playback_->pulseStream());
-
-            if (name && *name)
-                audioPref.setDevicePlayback(name);
-        }
-
-        delete playback_;
-        playback_ = NULL;
-    }
-
-    if (ringtone_) {
-        if (ringtone_->pulseStream()) {
-            const char *name = pa_stream_get_device_name(ringtone_->pulseStream());
-
-            if (name && *name)
-                audioPref.setDeviceRingtone(name);
-        }
-
-        delete ringtone_;
-        ringtone_ = NULL;
-    }
-
-    if (record_) {
-        if (record_->pulseStream()) {
-            const char *name = pa_stream_get_device_name(record_->pulseStream());
-
-            if (name && *name)
-                audioPref.setDeviceRecord(name);
-        }
-
-        delete record_;
-        record_ = NULL;
-    }
+    cleanupStream(playback_);
+    cleanupStream(ringtone_);
+    cleanupStream(record_);
 }
 
 void PulseLayer::startStream()
@@ -325,7 +344,7 @@ void PulseLayer::writeToSpeaker()
 
     notifyIncomingCall();
 
-    size_t urgentBytes = urgentRingBuffer_.AvailForGet(MainBuffer::DEFAULT_ID);
+    size_t urgentBytes = urgentRingBuffer_.availableForGet(MainBuffer::DEFAULT_ID);
 
     if (urgentBytes > writableBytes)
         urgentBytes = writableBytes;
@@ -333,8 +352,8 @@ void PulseLayer::writeToSpeaker()
     void *data = 0;
     if (urgentBytes) {
         pa_stream_begin_write(s, &data, &urgentBytes);
-        urgentRingBuffer_.Get(data, urgentBytes, MainBuffer::DEFAULT_ID);
-        applyGain(static_cast<SFLDataFormat *>(data), urgentBytes / sizeof(SFLDataFormat), getPlaybackGain());  
+        urgentRingBuffer_.get(data, urgentBytes, MainBuffer::DEFAULT_ID);
+        applyGain(static_cast<SFLDataFormat *>(data), urgentBytes / sizeof(SFLDataFormat), getPlaybackGain());
         pa_stream_write(s, data, urgentBytes, NULL, 0, PA_SEEK_RELATIVE);
         // Consume the regular one as well (same amount of bytes)
         Manager::instance().getMainBuffer()->discard(urgentBytes, MainBuffer::DEFAULT_ID);
@@ -356,7 +375,7 @@ void PulseLayer::writeToSpeaker()
 
     flushUrgent(); // flush remaining samples in _urgentRingBuffer
 
-    size_t availSamples = Manager::instance().getMainBuffer()->availForGet(MainBuffer::DEFAULT_ID) / sizeof(SFLDataFormat);
+    size_t availSamples = Manager::instance().getMainBuffer()->availableForGet(MainBuffer::DEFAULT_ID) / sizeof(SFLDataFormat);
 
     if (availSamples == 0) {
         pa_stream_begin_write(s, &data, &writableBytes);
@@ -479,71 +498,60 @@ void PulseLayer::ringtoneToSpeaker()
     pa_stream_write(s, data, bytes, NULL, 0, PA_SEEK_RELATIVE);
 }
 
-void PulseLayer::context_changed_callback(pa_context* c, pa_subscription_event_type_t t, uint32_t idx UNUSED, void *userdata)
+void
+PulseLayer::context_changed_callback(pa_context* c,
+                                     pa_subscription_event_type_t type,
+                                     uint32_t idx UNUSED, void *userdata)
 {
     PulseLayer *context = static_cast<PulseLayer*>(userdata);
-    switch (t) {
+    switch (type & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
+        pa_operation *op;
+
         case PA_SUBSCRIPTION_EVENT_SINK:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SINK");
-            context->sinkList_.clear();
-            pa_context_get_sink_info_list(c, sink_input_info_callback,  userdata);
+            switch (type & PA_SUBSCRIPTION_EVENT_TYPE_MASK) {
+                case PA_SUBSCRIPTION_EVENT_NEW:
+                case PA_SUBSCRIPTION_EVENT_REMOVE:
+                    DEBUG("Updating sink list");
+                    context->sinkList_.clear();
+                    op = pa_context_get_sink_info_list(c, sink_input_info_callback, userdata);
+                    if (op != NULL)
+                        pa_operation_unref(op);
+                default:
+                    break;
+            }
             break;
+
         case PA_SUBSCRIPTION_EVENT_SOURCE:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SOURCE");
-            context->sourceList_.clear();
-            pa_context_get_source_info_list(c, source_input_info_callback, userdata);
-            break;
-        case PA_SUBSCRIPTION_EVENT_SINK_INPUT:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SINK_INPUT");
-            break;
-        case PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SOURCE_OUTPUT");
-            break;
-        case PA_SUBSCRIPTION_EVENT_MODULE:
-            DEBUG("PA_SUBSCRIPTION_EVENT_MODULE");
-            break;
-        case PA_SUBSCRIPTION_EVENT_CLIENT:
-            DEBUG("PA_SUBSCRIPTION_EVENT_CLIENT");
-            break;
-        case PA_SUBSCRIPTION_EVENT_SAMPLE_CACHE:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SAMPLE_CACHE");
-            break;
-        case PA_SUBSCRIPTION_EVENT_SERVER:
-            DEBUG("PA_SUBSCRIPTION_EVENT_SERVER");
-            break;
-        case PA_SUBSCRIPTION_EVENT_CARD:
-            DEBUG("PA_SUBSCRIPTION_EVENT_CARD");
-            break;
-        case PA_SUBSCRIPTION_EVENT_FACILITY_MASK:
-            DEBUG("PA_SUBSCRIPTION_EVENT_FACILITY_MASK");
-            break;
-        case PA_SUBSCRIPTION_EVENT_CHANGE:
-            DEBUG("PA_SUBSCRIPTION_EVENT_CHANGE");
-            break;
-        case PA_SUBSCRIPTION_EVENT_REMOVE:
-            DEBUG("PA_SUBSCRIPTION_EVENT_REMOVE");
-            context->sinkList_.clear();
-            context->sourceList_.clear();
-            pa_context_get_sink_info_list(c, sink_input_info_callback, userdata);
-            pa_context_get_source_info_list(c, source_input_info_callback, userdata);
-            break;
-        case PA_SUBSCRIPTION_EVENT_TYPE_MASK:
-            DEBUG("PA_SUBSCRIPTION_EVENT_TYPE_MASK");
+            switch (type & PA_SUBSCRIPTION_EVENT_TYPE_MASK) {
+                case PA_SUBSCRIPTION_EVENT_NEW:
+                case PA_SUBSCRIPTION_EVENT_REMOVE:
+                    DEBUG("Updating source list");
+                    context->sourceList_.clear();
+                    op = pa_context_get_source_info_list(c, source_input_info_callback, userdata);
+                    if (op != NULL)
+                        pa_operation_unref(op);
+                default:
+                    break;
+            }
             break;
         default:
-            DEBUG("Unknown event type %d", t);
-
+            DEBUG("Unhandled event type 0x%x", type);
+            break;
     }
 }
 
+
 void PulseLayer::source_input_info_callback(pa_context *c UNUSED, const pa_source_info *i, int eol, void *userdata)
 {
     char s[PA_SAMPLE_SPEC_SNPRINT_MAX], cv[PA_CVOLUME_SNPRINT_MAX], cm[PA_CHANNEL_MAP_SNPRINT_MAX];
+    PulseLayer *context = static_cast<PulseLayer*>(userdata);
 
-    if (eol)
+    if (eol) {
+        context->enumeratingSources_ = false;
         return;
+    }
 
-    DEBUG("Sink %u\n"
+    DEBUG("Source %u\n"
            "    Name: %s\n"
            "    Driver: %s\n"
            "    Description: %s\n"
@@ -568,15 +576,18 @@ void PulseLayer::source_input_info_callback(pa_context *c UNUSED, const pa_sourc
            i->flags & PA_SOURCE_LATENCY ? "LATENCY " : "",
            i->flags & PA_SOURCE_HARDWARE ? "HARDWARE" : "");
 
-    static_cast<PulseLayer *>(userdata)->sourceList_.push_back(i->name);
+    context->sourceList_.push_back(i->name);
 }
 
 void PulseLayer::sink_input_info_callback(pa_context *c UNUSED, const pa_sink_info *i, int eol, void *userdata)
 {
     char s[PA_SAMPLE_SPEC_SNPRINT_MAX], cv[PA_CVOLUME_SNPRINT_MAX], cm[PA_CHANNEL_MAP_SNPRINT_MAX];
+    PulseLayer *context = static_cast<PulseLayer*>(userdata);
 
-    if (eol)
+    if (eol) {
+        context->enumeratingSinks_ = false;
         return;
+    }
 
     DEBUG("Sink %u\n"
           "    Name: %s\n"
@@ -603,6 +614,41 @@ void PulseLayer::sink_input_info_callback(pa_context *c UNUSED, const pa_sink_in
           i->flags & PA_SINK_LATENCY ? "LATENCY " : "",
           i->flags & PA_SINK_HARDWARE ? "HARDWARE" : "");
 
-    static_cast<PulseLayer *>(userdata)->sinkList_.push_back(i->name);
+    context->sinkList_.push_back(i->name);
 }
 
+void PulseLayer::updatePreference(AudioPreference &preference, int index, PCMType type)
+{
+    const std::string devName(getAudioDeviceName(index, type));
+    switch (type) {
+        case SFL_PCM_PLAYBACK:
+            DEBUG("setting %s for playback", devName.c_str());
+            preference.setPulseDevicePlayback(devName);
+            break;
+        case SFL_PCM_CAPTURE:
+            DEBUG("setting %s for capture", devName.c_str());
+            preference.setPulseDeviceRecord(devName);
+            break;
+        case SFL_PCM_RINGTONE:
+            DEBUG("setting %s for ringer", devName.c_str());
+            preference.setPulseDeviceRingtone(devName);
+            break;
+        default:
+            break;
+    }
+}
+
+int PulseLayer::getIndexCapture() const
+{
+    return getAudioDeviceIndex(preference_.getPulseDeviceRecord());
+}
+
+int PulseLayer::getIndexPlayback() const
+{
+    return getAudioDeviceIndex(preference_.getPulseDevicePlayback());
+}
+
+int PulseLayer::getIndexRingtone() const
+{
+    return getAudioDeviceIndex(preference_.getPulseDeviceRingtone());
+}
