@@ -37,7 +37,9 @@
 #include "sip/sipvoiplink.h"
 #include "audio/audiolayer.h"
 #include "audio/audiortp/audio_rtp_factory.h"
+#if HAVE_ZRTP
 #include "audio/audiortp/audio_zrtp_session.h"
+#endif
 
 #include "logger.h"
 #include "manager.h"
@@ -139,7 +141,7 @@ void CallManager::setVolume(const std::string& device, const double& value)
 
     DEBUG("DBUS set volume for %s: %f", device.c_str(), value);
 
-    if (device == "speaker") { 
+    if (device == "speaker") {
         audiolayer->setPlaybackGain((int)(value * 100.0));
     } else if (device == "mic") {
         audiolayer->setCaptureGain((int)(value * 100.0));
@@ -173,7 +175,7 @@ CallManager::joinParticipant(const std::string& sel_callID, const std::string& d
 }
 
 void
-CallManager::createConfFromParticipantList(const std::vector< std::string >& participants)
+CallManager::createConfFromParticipantList(const std::vector<std::string>& participants)
 {
     Manager::instance().createConfFromParticipantList(participants);
 }
@@ -214,19 +216,19 @@ CallManager::unholdConference(const std::string& confID)
     Manager::instance().unHoldConference(confID);
 }
 
-std::map< std::string, std::string >
+std::map<std::string, std::string>
 CallManager::getConferenceDetails(const std::string& callID)
 {
     return Manager::instance().getConferenceDetails(callID);
 }
 
-std::vector< std::string >
+std::vector<std::string>
 CallManager::getConferenceList()
 {
     return Manager::instance().getConferenceList();
 }
 
-std::vector< std::string >
+std::vector<std::string>
 CallManager::getParticipantList(const std::string& confID)
 {
     return Manager::instance().getParticipantList(confID);
@@ -256,6 +258,12 @@ CallManager::setRecording(const std::string& callID)
     Manager::instance().setRecordingCall(callID);
 }
 
+void
+CallManager::recordPlaybackSeek(const double& value)
+{
+    Manager::instance().recordingPlaybackSeek(value);
+}
+
 bool
 CallManager::getIsRecording(const std::string& callID)
 {
@@ -264,7 +272,7 @@ CallManager::getIsRecording(const std::string& callID)
 
 std::string CallManager::getCurrentAudioCodecName(const std::string& callID)
 {
-    return Manager::instance().getCurrentCodecName(callID).c_str();
+    return Manager::instance().getCurrentAudioCodecName(callID);
 }
 
 std::map<std::string, std::string>
@@ -301,6 +309,7 @@ CallManager::startTone(const int32_t& start , const int32_t& type)
 // for conferencing in order to get
 // the right pointer for the given
 // callID.
+#if HAVE_ZRTP
 sfl::AudioZrtpSession *
 CallManager::getAudioZrtpSession(const std::string& callID)
 {
@@ -324,76 +333,90 @@ CallManager::getAudioZrtpSession(const std::string& callID)
 
     return zSession;
 }
+#endif
 
 void
 CallManager::setSASVerified(const std::string& callID)
 {
+#if HAVE_ZRTP
     try {
         sfl::AudioZrtpSession * zSession;
         zSession = getAudioZrtpSession(callID);
         zSession->SASVerified();
     } catch (...) {
     }
+#else
+    ERROR("No zrtp support for %s, please recompile SFLphone with zrtp", callID.c_str());
+#endif
 }
 
 void
 CallManager::resetSASVerified(const std::string& callID)
 {
+#if HAVE_ZRTP
     try {
         sfl::AudioZrtpSession * zSession;
         zSession = getAudioZrtpSession(callID);
         zSession->resetSASVerified();
     } catch (...) {
     }
+#else
+    ERROR("No zrtp support for %s, please recompile SFLphone with zrtp", callID.c_str());
+#endif
 }
 
 void
 CallManager::setConfirmGoClear(const std::string& callID)
 {
+#if HAVE_ZRTP
     try {
         sfl::AudioZrtpSession * zSession;
         zSession = getAudioZrtpSession(callID);
         zSession->goClearOk();
     } catch (...) {
     }
+#else
+    ERROR("No zrtp support for %s, please recompile SFLphone with zrtp", callID.c_str());
+#endif
 }
 
 void
 CallManager::requestGoClear(const std::string& callID)
 {
+#if HAVE_ZRTP
     try {
         sfl::AudioZrtpSession * zSession;
         zSession = getAudioZrtpSession(callID);
         zSession->requestGoClear();
     } catch (...) {
     }
+#else
+    ERROR("No zrtp support for %s, please recompile SFLphone with zrtp", callID.c_str());
+#endif
 }
 
 void
 CallManager::acceptEnrollment(const std::string& callID, const bool& accepted)
 {
+#if HAVE_ZRTP
     try {
         sfl::AudioZrtpSession * zSession;
         zSession = getAudioZrtpSession(callID);
         zSession->acceptEnrollment(accepted);
     } catch (...) {
     }
-}
-
-void
-CallManager::setPBXEnrollment(const std::string& callID, const bool& yesNo)
-{
-    try {
-        sfl::AudioZrtpSession * zSession;
-        zSession = getAudioZrtpSession(callID);
-        zSession->setPBXEnrollment(yesNo);
-    } catch (...) {
-    }
+#else
+    ERROR("No zrtp support for %s, please recompile SFLphone with zrtp", callID.c_str());
+#endif
 }
 
 void
 CallManager::sendTextMessage(const std::string& callID, const std::string& message)
 {
+#if HAVE_INSTANT_MESSAGING
     if (!Manager::instance().sendTextMessage(callID, message, "Me"))
         throw CallManagerException();
+#else
+    ERROR("Could not send \"%s\" text message to %s since SFLphone daemon does not support it, please recompile with instant messaging support", message.c_str(), callID.c_str());
+#endif
 }
