@@ -28,6 +28,10 @@
  *  as that of the covered work.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <cstdlib>
 #include "dbusmanager.h"
 #include "global.h"
@@ -39,27 +43,47 @@
 #include "configurationmanager.h"
 #include "networkmanager.h"
 
+#ifdef SFL_VIDEO
+#include "dbus/video_controls.h"
+#endif
+
 DBusManager::DBusManager() : callManager_(0)
     , configurationManager_(0)
     , instanceManager_(0)
     , dispatcher_()
-#if USE_NETWORKMANAGER
+#ifdef SFL_VIDEO
+    , videoControls_(0)
+#endif
+#ifdef USE_NETWORKMANAGER
     , networkManager_(0)
 #endif
 {
     try {
+        DEBUG("DBUS init threading");
         DBus::_init_threading();
+        DEBUG("DBUS instantiate default dispatcher");
         DBus::default_dispatcher = &dispatcher_;
 
+        DEBUG("DBUS session connection to session bus");
         DBus::Connection sessionConnection(DBus::Connection::SessionBus());
+        DEBUG("DBUS request org.sflphone.SFLphone from session connection");
         sessionConnection.request_name("org.sflphone.SFLphone");
 
+        DEBUG("DBUS create call manager from session connection");
         callManager_ = new CallManager(sessionConnection);
+        DEBUG("DBUS create configuration manager from session connection");
         configurationManager_ = new ConfigurationManager(sessionConnection);
+        DEBUG("DBUS create instance manager from session connection");
         instanceManager_ = new Instance(sessionConnection);
 
+#ifdef SFL_VIDEO
+        videoControls_ = new VideoControls(sessionConnection);
+#endif
+
 #ifdef USE_NETWORKMANAGER
+        DEBUG("DBUS system connection to system bus");
         DBus::Connection systemConnection(DBus::Connection::SystemBus());
+        DEBUG("DBUS create the network manager from the system bus");
         networkManager_ = new NetworkManager(systemConnection, "/org/freedesktop/NetworkManager", "");
 #endif
 
@@ -67,12 +91,17 @@ DBusManager::DBusManager() : callManager_(0)
         ERROR("%s: %s, exiting\n", err.name(), err.what());
         ::exit(EXIT_FAILURE);
     }
+
+    DEBUG("DBUS registration done");
 }
 
 DBusManager::~DBusManager()
 {
 #ifdef USE_NETWORKMANAGER
     delete networkManager_;
+#endif
+#ifdef SFL_VIDEO
+    delete videoControls_;
 #endif
     delete instanceManager_;
     delete configurationManager_;
@@ -92,8 +121,7 @@ void DBusManager::exec()
     }
 }
 
-void
-DBusManager::exit()
+void DBusManager::exit()
 {
     try {
         dispatcher_.leave();
