@@ -37,6 +37,7 @@
 #include <string>
 #include <sstream>
 #include <iostream>
+#include <unistd.h>
 #include "fileutils.h"
 
 namespace fileutils {
@@ -68,10 +69,17 @@ const char *get_program_dir()
     return program_dir;
 }
 
+//TODO it is faking this, implement proper system 
+const char *get_data_dir()
+{
+    std::string path = std::string(get_program_dir()) + "/../../share/sflphone/ringtones/";
+    return path.c_str();
+}
+
 bool create_pidfile()
 {
-    const char * const xdg_env = XDG_CACHE_HOME;
-    std::string path = xdg_env ? xdg_env : std::string(HOMEDIR) + DIR_SEPARATOR_STR ".cache/";
+    std::string xdg_env(XDG_CACHE_HOME);
+    std::string path = (not xdg_env.empty()) ? xdg_env : std::string(HOMEDIR) + DIR_SEPARATOR_STR ".cache/";
 
     if (!check_dir(path.c_str()))
         return false;
