@@ -166,9 +166,9 @@ IAXVoIPLink::sendAudioFromMic()
 
         // we have to get 20ms of data from the mic *20/1000 = /50
         // rate/50 shall be lower than IAX__20S_48KHZ_MAX
-        int bytesNeeded = mainBufferSampleRate * 20 / 1000 * sizeof(SFLDataFormat);
+        const size_t bytesNeeded = mainBufferSampleRate * 20 / 1000 * sizeof(SFLDataFormat);
 
-        if (Manager::instance().getMainBuffer()->availForGet(currentCall->getCallId()) < bytesNeeded)
+        if (Manager::instance().getMainBuffer()->availableForGet(currentCall->getCallId()) < bytesNeeded)
             continue;
 
         // Get bytes from micRingBuffer to data_from_mic
@@ -229,7 +229,7 @@ IAXVoIPLink::sendRegister(Account *a)
     if (regSession_) {
         iax_register(regSession_, account->getHostname().data(), account->getUsername().data(), account->getPassword().data(), 120);
         nextRefreshStamp_ = time(NULL) + 10;
-        account->setRegistrationState(Trying);
+        account->setRegistrationState(TRYING);
     }
 }
 
@@ -244,7 +244,7 @@ IAXVoIPLink::sendUnregister(Account *a)
 
     nextRefreshStamp_ = 0;
 
-    dynamic_cast<IAXAccount*>(a)->setRegistrationState(Unregistered);
+    dynamic_cast<IAXAccount*>(a)->setRegistrationState(UNREGISTERED);
 }
 
 Call*
@@ -399,6 +399,7 @@ IAXVoIPLink::carryingDTMFdigits(const std::string& id, char code)
     }
 }
 
+#if HAVE_INSTANT_MESSAGING
 void
 IAXVoIPLink::sendTextMessage(const std::string& callID,
                              const std::string& message,
@@ -411,9 +412,17 @@ IAXVoIPLink::sendTextMessage(const std::string& callID,
         sfl::InstantMessaging::send_iax_message(call->session, callID, message.c_str());
     }
 }
+#endif
 
 std::string
-IAXVoIPLink::getCurrentCodecName(Call *c) const
+IAXVoIPLink::getCurrentVideoCodecName(Call * /*call*/) const
+{
+    // FIXME: Video not supported for IAX yet
+    return "";
+}
+
+std::string
+IAXVoIPLink::getCurrentAudioCodecName(Call *c) const
 {
     IAXCall *call = dynamic_cast<IAXCall*>(c);
     sfl::Codec *audioCodec = Manager::instance().audioCodecFactory.getCodec(call->getAudioCodec());
@@ -514,7 +523,9 @@ IAXVoIPLink::iaxHandleCallEvent(iax_event* event, IAXCall* call)
             break;
 
         case IAX_EVENT_TEXT:
+#if HAVE_INSTANT_MESSAGING
             Manager::instance().incomingMessage(call->getCallId(), call->getPeerNumber(), std::string((const char*) event->data));
+#endif
             break;
 
         case IAX_EVENT_RINGA:
@@ -593,7 +604,7 @@ void IAXVoIPLink::iaxHandleRegReply(iax_event* event)
     iax_destroy(regSession_);
     regSession_ = NULL;
 
-    account->setRegistrationState((event->etype == IAX_EVENT_REGREJ) ? ErrorAuth : Registered);
+    account->setRegistrationState((event->etype == IAX_EVENT_REGREJ) ? ERROR_AUTH : REGISTERED);
 
     if (event->etype == IAX_EVENT_REGACK)
         nextRefreshStamp_ = time(NULL) + (event->ies.refresh ? event->ies.refresh : 60);
