@@ -37,7 +37,6 @@
 #include <iostream>
 #include <getopt.h>
 #include "fileutils.h"
-#include "dbus/dbusmanager.h"
 #include "logger.h"
 #include "manager.h"
 
@@ -122,9 +121,30 @@ namespace {
     }
 }
 
+namespace {
+    void signal_handler(int code)
+    {
+        std::cerr << "Caught signal " << strsignal(code) << ", terminating..." << std::endl;
+        Manager::instance().finish();
+    }
+}
+
 int main(int argc, char *argv [])
 {
-    fileutils::set_program_dir(argv[0]);
+    // TODO: Block signals for all threads but the main thread, decide how/if we should
+    // handle other signals
+    signal(SIGINT, signal_handler);
+    signal(SIGHUP, signal_handler);
+    signal(SIGTERM, signal_handler);
+
+    // make a copy as we don't want to modify argv[0], copy it to a vector to
+    // guarantee that memory is correctly managed/exception safe
+    std::string programName(argv[0]);
+    std::vector<char> writable(programName.size() + 1);
+    std::copy(programName.begin(), programName.end(), writable.begin());
+
+    fileutils::set_program_dir(&*writable.begin());
+
     print_title();
     if (parse_args(argc, argv))
         return 0;
@@ -143,8 +163,11 @@ int main(int argc, char *argv [])
         return 1;
     }
 
-    DEBUG("Starting DBus event loop");
-    Manager::instance().getDbusManager()->exec();
+#ifdef SFL_VIDEO
+    WARN("Built with video support");
+#endif
+
+    Manager::instance().run();
     Manager::instance().saveHistory();
 
     return 0;
