@@ -35,6 +35,10 @@
 #ifndef MANAGER_IMPL_H_
 #define MANAGER_IMPL_H_
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <string>
 #include <vector>
 #include <set>
@@ -48,6 +52,7 @@
 #include "call.h"
 #include "conference.h"
 
+#include "audio/audiolayer.h"
 #include "audio/sound/tone.h"  // for Tone::TONEID declaration
 #include "audio/codecs/audiocodecfactory.h"
 
@@ -131,9 +136,14 @@ class ManagerImpl {
         void init(const std::string &config_file);
 
         /**
-         * Terminate all thread (sound, link) and unload AccountMap
+         * Enter Dbus mainloop
          */
-        void terminate();
+        void run();
+
+        /*
+         * Terminate all threads and exit DBus loop
+         */
+        void finish();
 
         /**
          * Accessor to audiodriver.
@@ -375,6 +385,7 @@ class ManagerImpl {
          */
         void peerHungupCall(const std::string& id);
 
+#if HAVE_INSTANT_MESSAGING
         /**
          * Notify the client with an incoming message
          * @param accountId	The account identifier
@@ -390,6 +401,7 @@ class ManagerImpl {
         * @param from	        The sender of this message (could be another participant of a conference)
          */
         bool sendTextMessage(const std::string& callID, const std::string& message, const std::string& from);
+#endif // HAVE_INSTANT_MESSAGING
 
         /**
          * Notify the client he has voice mails
@@ -407,6 +419,16 @@ class ManagerImpl {
          */
         void sendRegister(const std::string& accountId, bool enable);
 
+        /**
+         * Register all account in accountMap_
+         */
+        void registerAllAccounts();
+
+        /**
+         * Unregister all account in accountMap_
+         */
+        void unregisterAllAccounts();
+
         /**
          * Get account list
          * @return std::vector<std::string> A list of accoundIDs
@@ -489,7 +511,8 @@ class ManagerImpl {
          * @param call id
          * @return std::string The codec name
          */
-        std::string getCurrentCodecName(const std::string& id);
+        std::string getCurrentAudioCodecName(const std::string& id);
+        std::string getCurrentVideoCodecName(const std::string& id);
 
         /**
          * Set input audio plugin
@@ -502,7 +525,7 @@ class ManagerImpl {
              * @param index The index of the soundcard
              * @param the type of stream, either SFL_PCM_PLAYBACK, SFL_PCM_CAPTURE, SFL_PCM_RINGTONE
              */
-        void setAudioDevice(const int index, const int streamType);
+        void setAudioDevice(int index, AudioLayer::PCMType streamType);
 
         /**
          * Get list of supported audio output device
@@ -630,6 +653,8 @@ class ManagerImpl {
              */
         bool startRecordedFilePlayback(const std::string&);
 
+        void recordingPlaybackSeek(const double value);
+
         /**
          * Stop playback of recorded file
          * @param File of the file to stop
@@ -714,12 +739,6 @@ class ManagerImpl {
          */
         int32_t getMailNotify() const;
 
-        /**
-         * Get the list of the active codecs
-         * @return std::vector< ::std::string >  The list of active codecs
-         */
-        std::vector<std::string> getActiveCodecList() const;
-
         /**
          * Change a specific value in the configuration tree.
          * This value will then be saved in the user config file sflphonedrc
@@ -841,6 +860,12 @@ class ManagerImpl {
         const AudioCodecFactory audioCodecFactory;
 
     private:
+
+        /**
+         * Get the Call referred to by callID. If the Call does not exist, return NULL
+         */
+        Call *getCallFromCallID(const std::string &callID);
+
         /**
          * Play the dtmf-associated sound
          * @param code  The pressed key
@@ -853,7 +878,7 @@ class ManagerImpl {
          * @param current call id
          * @param conference pointer
          */
-        void processRemainingParticipants(const std::string &current_call_id, Conference * &conf);
+        void processRemainingParticipants(Conference &conf);
 
         /**
          * Create config directory in home user and return configuration file path
@@ -865,6 +890,11 @@ class ManagerImpl {
          */
         void initZeroconf();
 
+        /**
+         * Set current call ID to empty string
+         */
+        void unsetCurrentCall();
+
         /**
          * Switch of current call id
          * @param id The new callid
@@ -970,11 +1000,6 @@ class ManagerImpl {
          *Contains a list of account (sip, aix, etc) and their respective voiplink/calls */
         AccountMap accountMap_;
 
-        /**
-         * Unregister all account in accountMap_
-         */
-        void unregisterAllAccounts();
-
         /**
          * Load the account map from configuration
          */
@@ -1039,6 +1064,12 @@ class ManagerImpl {
             return &dbus_;
         }
 
+#ifdef SFL_VIDEO
+        VideoControls * getVideoControls() {
+            return dbus_.getVideoControls();
+        }
+#endif
+
         /**
         * Tell if an account exists
         * @param accountID account ID check
@@ -1047,7 +1078,7 @@ class ManagerImpl {
         */
         bool accountExists(const std::string& accountID);
 
-        std::vector<std::map<std::string, std::string> > getHistory() const;
+        std::vector<std::map<std::string, std::string> > getHistory();
         void clearHistory();
 
         /**
@@ -1070,7 +1101,7 @@ class ManagerImpl {
          * @param accountID	  Account ID to get
          * @return VoIPLink*   The voip link from the account pointer or 0
          */
-        VoIPLink* getAccountLink(const std::string& accountID="");
+        VoIPLink* getAccountLink(const std::string& accountID);
 
         std::string getAccountIdFromNameAndServer(const std::string& userName, const std::string& server) const;
 
@@ -1097,5 +1128,6 @@ class ManagerImpl {
           * TODO: move this to ConfigurationManager
           */
         History history_;
+        bool finished_;
 };
 #endif // MANAGER_IMPL_H_
