@@ -312,66 +312,66 @@ class AudioPreference : public Serializable {
         virtual void unserialize(const Conf::MappingNode &map);
 
         // alsa preference
-        int getCardin() const {
-            return cardin_;
+        int getAlsaCardin() const {
+            return alsaCardin_;
         }
-        void setCardin(int c) {
-            cardin_ = c;
+        void setAlsaCardin(int c) {
+            alsaCardin_ = c;
         }
 
-        int getCardout() const {
-            return cardout_;
+        int getAlsaCardout() const {
+            return alsaCardout_;
         }
 
-        void setCardout(int c) {
-            cardout_ = c;
+        void setAlsaCardout(int c) {
+            alsaCardout_ = c;
         }
 
-        int getCardring() const {
-            return cardring_;
+        int getAlsaCardring() const {
+            return alsaCardring_;
         }
 
-        void setCardring(int c) {
-            cardring_ = c;
+        void setAlsaCardring(int c) {
+            alsaCardring_ = c;
         }
 
-        std::string getPlugin() const {
-            return plugin_;
+        std::string getAlsaPlugin() const {
+            return alsaPlugin_;
         }
 
-        void setPlugin(const std::string &p) {
-            plugin_ = p;
+        void setAlsaPlugin(const std::string &p) {
+            alsaPlugin_ = p;
         }
 
-        int getSmplrate() const {
-            return smplrate_;
+        int getAlsaSmplrate() const {
+            return alsaSmplrate_;
         }
-        void setSmplrate(int r) {
-            smplrate_ = r;
+        void setAlsaSmplrate(int r) {
+            alsaSmplrate_ = r;
         }
 
         //pulseaudio preference
-        std::string getDevicePlayback() const {
-            return devicePlayback_;
+        std::string getPulseDevicePlayback() const {
+            return pulseDevicePlayback_;
         }
 
-        void setDevicePlayback(const std::string &p) {
-            devicePlayback_ = p;
+        void setPulseDevicePlayback(const std::string &p) {
+            pulseDevicePlayback_ = p;
         }
 
-        std::string getDeviceRecord() const {
-            return deviceRecord_;
+        std::string getPulseDeviceRecord() const {
+            return pulseDeviceRecord_;
         }
-        void setDeviceRecord(const std::string &r) {
-            deviceRecord_ = r;
+        void setPulseDeviceRecord(const std::string &r) {
+            pulseDeviceRecord_ = r;
         }
 
-        std::string getDeviceRingtone() const {
-            return deviceRingtone_;
+        std::string getPulseDeviceRingtone() const {
+            return pulseDeviceRingtone_;
         }
 
-        void setDeviceRingtone(const std::string &r) {
-            deviceRingtone_ = r;
+        void setPulseDeviceRingtone(const std::string &r) {
+            pulseDeviceRingtone_ = r;
         }
 
         // general preference
@@ -440,16 +440,16 @@ class AudioPreference : public Serializable {
         std::string audioApi_;
 
         // alsa preference
-        int cardin_;
-        int cardout_;
-        int cardring_;
-        std::string plugin_;
-        int smplrate_;
+        int alsaCardin_;
+        int alsaCardout_;
+        int alsaCardring_;
+        std::string alsaPlugin_;
+        int alsaSmplrate_;
 
         //pulseaudio preference
-        std::string devicePlayback_;
-        std::string deviceRecord_;
-        std::string deviceRingtone_;
+        std::string pulseDevicePlayback_;
+        std::string pulseDeviceRecord_;
+        std::string pulseDeviceRingtone_;
 
         // general preference
         std::string recordpath_; //: /home/msavard/Bureau
