@@ -57,7 +57,7 @@ gboolean sflphone_init(GError **error);
 /**
  * Steps when closing the application.  Will ask for confirmation if a call is in progress.
  */
-void sflphone_quit();
+void sflphone_quit(gboolean force_quit);
 
 /**
  * Hang up / refuse the current call
@@ -192,8 +192,6 @@ void sflphone_fill_codec_list_per_account(account_t *);
 
 void sflphone_add_participant();
 
-void sflphone_record(callable_obj_t *c);
-
 void sflphone_rec_call(void);
 
 void sflphone_mute_call(void);
