@@ -43,17 +43,24 @@
 #include "shortcuts.h"
 #include "history.h"
 
+static void
+signal_handler(int code)
+{
+    // Unset signal handlers
+    signal(SIGHUP, SIG_DFL);
+    signal(SIGINT, SIG_DFL);
+    signal(SIGTERM, SIG_DFL);
+
+    printf("Caught signal %s, quitting...\n", strsignal(code));
+    sflphone_quit(TRUE);
+}
+
 int
 main(int argc, char *argv[])
 {
-    GError *error = NULL;
-    // Handle logging
-    int i;
-
-    // Check arguments if debug mode is activated
-    for (i = 0; i < argc; i++)
-        if (g_strcmp0(argv[i], "--debug") == 0)
-            set_log_level(LOG_DEBUG);
+    signal(SIGINT, signal_handler);
+    signal(SIGHUP, signal_handler);
+    signal(SIGTERM, signal_handler);
 
 #if !GTK_CHECK_VERSION(2,32,0)
     g_thread_init(NULL);
@@ -64,6 +71,13 @@ main(int argc, char *argv[])
     // Start GTK application
     gtk_init(&argc, &argv);
 
+    // Handle logging
+
+    // Check arguments if debug mode is activated
+    for (int i = 0; i < argc; i++)
+        if (g_strcmp0(argv[i], "--debug") == 0)
+            set_log_level(LOG_DEBUG);
+
     g_print("%s %s\n", PACKAGE, VERSION);
     g_print("\nCopyright (c) 2005 - 2012 Savoir-faire Linux Inc.\n\n");
     g_print("This is free software.  You may redistribute copies of it under the terms of\n" \
@@ -82,9 +96,10 @@ main(int argc, char *argv[])
     srand(time(NULL));
 
     // Internationalization
-    bindtextdomain("sflphone-client-gnome", LOCALEDIR);
-    textdomain("sflphone-client-gnome");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
 
+    GError *error = NULL;
     if (!sflphone_init(&error)) {
         ERROR("%s", error->message);
         GtkWidget *dialog = gtk_message_dialog_new(
@@ -126,6 +141,7 @@ main(int argc, char *argv[])
 
     gtk_main();
 
+    codecs_unload();
     shortcuts_destroy_bindings();
 
     eel_gconf_global_client_free();
