@@ -35,6 +35,7 @@
 #endif
 
 #include "gtk2_wrappers.h"
+#include "account_schema.h"
 #include "actions.h"
 #include "dbus.h"
 #include "calltree.h"
@@ -52,6 +53,8 @@
 #include "unused.h"
 #include "config/audioconf.h"
 #include "str_utils.h"
+#include "seekslider.h"
+#include "messaging/message_tab.h"
 
 #include "eel-gconf-extensions.h"
 
@@ -71,10 +74,13 @@ static GtkWidget *dialpad;
 static GtkWidget *speaker_control;
 static GtkWidget *mic_control;
 static GtkWidget *statusBar;
+static GtkWidget *seekslider = NULL;
 
 static gchar *status_current_message;
 
-static gboolean focus_is_on_searchbar;
+static gboolean focus_is_on_searchbar = FALSE;
+
+static gboolean pause_grabber = FALSE;
 
 void
 focus_on_searchbar_out()
@@ -88,6 +94,17 @@ focus_on_searchbar_in()
     focus_is_on_searchbar = TRUE;
 }
 
+/**
+ * Save the vpaned size
+ */
+static void
+on_messaging_paned_position_change(GtkPaned* paned, GtkScrollType scroll_type UNUSED,gpointer user_data UNUSED)
+{
+    int height = gtk_paned_get_position(paned);
+    eel_gconf_set_integer(CONF_MESSAGING_HEIGHT, height);
+    set_message_tab_height(paned,height);
+}
+
 /**
  * Handle main window resizing
  */
@@ -96,6 +113,10 @@ static gboolean window_configure_cb(GtkWidget *win UNUSED, GdkEventConfigure *ev
     eel_gconf_set_integer(CONF_MAIN_WINDOW_WIDTH, event->width);
     eel_gconf_set_integer(CONF_MAIN_WINDOW_HEIGHT, event->height);
 
+    gint height = 0;
+    gint width  = 0;
+    gtk_widget_get_size_request(get_tab_box(),&width,&height);
+
     int pos_x, pos_y;
     gtk_window_get_position(GTK_WINDOW(window), &pos_x, &pos_y);
     eel_gconf_set_integer(CONF_MAIN_WINDOW_POSITION_X, pos_x);
@@ -104,7 +125,6 @@ static gboolean window_configure_cb(GtkWidget *win UNUSED, GdkEventConfigure *ev
     return FALSE;
 }
 
-
 /**
  * Minimize the main window.
  */
@@ -115,7 +135,7 @@ on_delete(GtkWidget * widget UNUSED, gpointer data UNUSED)
         gtk_widget_hide(get_main_window());
         set_minimized(TRUE);
     } else
-        sflphone_quit();
+        sflphone_quit(FALSE);
 
     return TRUE;
 }
@@ -146,42 +166,61 @@ main_window_ask_quit()
 static gboolean
 on_key_released(GtkWidget *widget UNUSED, GdkEventKey *event, gpointer user_data UNUSED)
 {
-    if (focus_is_on_searchbar)
-        return TRUE;
+    if (!pause_grabber) {
+        if (focus_is_on_searchbar)
+           return TRUE;
 
-    if (event->keyval == GDK_KEY_Return) {
-        if (active_calltree_tab == current_calls_tab) {
-            sflphone_keypad(event->keyval, event->string);
-            return TRUE;
-        } else if (active_calltree_tab == history_tab)
-            return FALSE;
+        if (event->keyval == GDK_KEY_Return) {
+           if (calltab_has_name(active_calltree_tab, CURRENT_CALLS)) {
+                 sflphone_keypad(event->keyval, event->string);
+                 return TRUE;
+           } else if (calltab_has_name(active_calltree_tab, HISTORY))
+                 return FALSE;
+        }
+
+        // If a modifier key is pressed, it's a shortcut, pass along
+        if (event->state & GDK_CONTROL_MASK || event->state & GDK_MOD1_MASK ||
+                 event->keyval == '<' ||
+                 event->keyval == '>' ||
+                 event->keyval == '\"'||
+                 event->keyval == GDK_KEY_Tab ||
+                 event->keyval == GDK_KEY_Return ||
+                 event->keyval == GDK_KEY_Left ||
+                 event->keyval == GDK_KEY_Up ||
+                 event->keyval == GDK_KEY_Right ||
+                 event->keyval == GDK_KEY_Down ||
+                 (event->keyval >= GDK_KEY_F1 && event->keyval <= GDK_KEY_F12) ||
+                 event->keyval == ' ')
+           return FALSE;
+        else
+           sflphone_keypad(event->keyval, event->string);
+
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static void pack_main_window_start(GtkBox *box, GtkWidget *widget, gboolean expand, gboolean fill, guint padding)
+{
+    if(box == NULL) {
+        ERROR("Box is NULL while packing main window");
+        return;
     }
 
-    // If a modifier key is pressed, it's a shortcut, pass along
-    if (event->state & GDK_CONTROL_MASK || event->state & GDK_MOD1_MASK ||
-            event->keyval == '<' ||
-            event->keyval == '>' ||
-            event->keyval == '\"'||
-            event->keyval == GDK_KEY_Tab ||
-            event->keyval == GDK_KEY_Return ||
-            event->keyval == GDK_KEY_Left ||
-            event->keyval == GDK_KEY_Up ||
-            event->keyval == GDK_KEY_Right ||
-            event->keyval == GDK_KEY_Down ||
-            (event->keyval >= GDK_KEY_F1 && event->keyval <= GDK_KEY_F12) ||
-            event->keyval == ' ')
-        return FALSE;
-    else
-        sflphone_keypad(event->keyval, event->string);
+    if(widget == NULL) {
+        ERROR("Widget is NULL while packing the mainwindow");
+        return;
+    }
 
-    return TRUE;
+    GtkWidget *alignment =  gtk_alignment_new(0.0, 0.0, 1.0, 1.0);
+    gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 6, 6);
+    gtk_container_add(GTK_CONTAINER(alignment), widget);
+    gtk_box_pack_start(box, alignment, expand, fill, padding);
 }
 
 void
 create_main_window()
 {
-    focus_is_on_searchbar = FALSE;
-
     // Get configuration stored in gconf
     int width =  eel_gconf_get_integer(CONF_MAIN_WINDOW_WIDTH);
     int height =  eel_gconf_get_integer(CONF_MAIN_WINDOW_HEIGHT);
@@ -189,7 +228,9 @@ create_main_window()
     int position_y =  eel_gconf_get_integer(CONF_MAIN_WINDOW_POSITION_Y);
 
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
     gtk_container_set_border_width(GTK_CONTAINER(window), 0);
+
     gtk_window_set_title(GTK_WINDOW(window), "SFLphone VoIP Client");
     gtk_window_set_default_size(GTK_WINDOW(window), width, height);
     struct stat st;
@@ -198,6 +239,7 @@ create_main_window()
         gtk_window_set_default_icon_from_file(LOGO, NULL);
 
     gtk_window_set_position(GTK_WINDOW(window) , GTK_WIN_POS_MOUSE);
+    gtk_widget_set_name(window, "mainwindow");
 
     /* Connect the destroy event of the window with our on_destroy function
      * When the window is about to be destroyed we get a notificaiton and
@@ -212,42 +254,67 @@ create_main_window()
     g_signal_connect_object(G_OBJECT(window), "configure-event",
                             G_CALLBACK(window_configure_cb), NULL, 0);
 
-    gtk_widget_set_name(window, "mainwindow");
 
     ui_manager = uimanager_new();
-
     if (!ui_manager) {
         ERROR("Could not load xml GUI\n");
         exit(1);
     }
 
     /* Create an accel group for window's shortcuts */
-    gtk_window_add_accel_group(GTK_WINDOW(window),
-                               gtk_ui_manager_get_accel_group(ui_manager));
+    gtk_window_add_accel_group(GTK_WINDOW(window), gtk_ui_manager_get_accel_group(ui_manager));
 
-    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0 /*spacing*/);
-    subvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5 /*spacing*/);
+    /* Instantiate vbox, subvbox as homogeneous */
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    gtk_box_set_homogeneous(GTK_BOX(vbox), FALSE);
 
+    subvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
+    gtk_box_set_homogeneous(GTK_BOX(subvbox), FALSE);
+
+    /*Create the messaging tab container*/
+    GtkWidget *tab_widget = get_tab_box();
+    gtk_widget_show (tab_widget);
+
+    /* Populate the main window */
     GtkWidget *widget = create_menus(ui_manager);
-    gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE /*expand*/, TRUE /*fill*/,
-                       0 /*padding*/);
+    gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 0);
 
     widget = create_toolbar_actions(ui_manager);
-    // Do not override GNOME user settings
-    gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE /*expand*/, TRUE /*fill*/,
-                       0 /*padding*/);
+    pack_main_window_start(GTK_BOX(vbox), widget, FALSE, TRUE, 0);
 
-    gtk_box_pack_start(GTK_BOX(vbox), current_calls_tab->tree, TRUE /*expand*/,
-                       TRUE /*fill*/, 0 /*padding*/);
-    gtk_box_pack_start(GTK_BOX(vbox), history_tab->tree, TRUE /*expand*/,
-                       TRUE /*fill*/, 0 /*padding*/);
-    gtk_box_pack_start(GTK_BOX(vbox), contacts_tab->tree, TRUE /*expand*/,
-                       TRUE /*fill*/, 0 /*padding*/);
+    /* Setup call main widget*/
+#if GTK_MAJOR_VERSION == 2
+    GtkWidget *vpaned = gtk_vpaned_new();
+#else
+    GtkWidget *vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
+#endif
+    current_calls_tab->mainwidget = vpaned;
 
-    g_signal_connect_object(G_OBJECT(window), "configure-event",
-                            G_CALLBACK(window_configure_cb), NULL, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), subvbox, FALSE /*expand*/,
-                       FALSE /*fill*/, 0 /*padding*/);
+    int messaging_height = eel_gconf_get_integer(CONF_MESSAGING_HEIGHT);
+    set_message_tab_height(GTK_PANED(vpaned),messaging_height);
+
+    gtk_widget_show (vpaned);
+    gtk_box_pack_start(GTK_BOX(vbox), vpaned, TRUE, TRUE, 0);
+
+    /* Setup history main widget */
+    GtkWidget *history_vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    history_tab->mainwidget = history_vbox;
+    gtk_box_set_homogeneous(GTK_BOX(history_vbox), FALSE);
+    gtk_box_pack_start(GTK_BOX(history_vbox), history_tab->tree, TRUE, TRUE, 0);
+
+    /* Add tree views */
+    gtk_box_pack_start(GTK_BOX(vbox), contacts_tab->tree, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), history_vbox, TRUE, TRUE, 0);
+    gtk_paned_pack1 (GTK_PANED (vpaned), current_calls_tab->tree, TRUE, FALSE);
+    gtk_paned_pack2 (GTK_PANED (vpaned), tab_widget, FALSE, FALSE);
+
+    g_signal_connect(G_OBJECT(vpaned), "notify::position" , G_CALLBACK(on_messaging_paned_position_change), current_calls_tab);
+
+    /* Add playback scale and setup history tab */
+    seekslider = GTK_WIDGET(sfl_seekslider_new());
+    pack_main_window_start(GTK_BOX(history_vbox), seekslider, FALSE, TRUE, 0);
+
+    gtk_box_pack_start(GTK_BOX(vbox), subvbox, FALSE, FALSE, 0);
 
     speaker_control = create_slider("speaker");
     mic_control = create_slider("mic");
@@ -255,10 +322,8 @@ create_main_window()
     g_object_ref(mic_control);
 
     if (SHOW_VOLUME) {
-        gtk_box_pack_end(GTK_BOX(subvbox), speaker_control, FALSE /*expand*/,
-                          TRUE /*fill*/, 0 /*padding*/);
-        gtk_box_pack_end(GTK_BOX(subvbox), mic_control, FALSE /*expand*/,
-                         TRUE /*fill*/, 0 /*padding*/);
+        gtk_box_pack_end(GTK_BOX(subvbox), speaker_control, FALSE, TRUE, 0);
+        gtk_box_pack_end(GTK_BOX(subvbox), mic_control, FALSE, TRUE, 0);
         gtk_widget_show_all(speaker_control);
         gtk_widget_show_all(mic_control);
     } else {
@@ -268,25 +333,31 @@ create_main_window()
 
     if (eel_gconf_get_boolean(CONF_SHOW_DIALPAD)) {
         dialpad = create_dialpad();
-        gtk_box_pack_end(GTK_BOX(subvbox), dialpad, FALSE /*expand*/, TRUE /*fill*/, 0 /*padding*/);
+        gtk_box_pack_end(GTK_BOX(subvbox), dialpad, FALSE, TRUE, 0);
         gtk_widget_show_all(dialpad);
     }
 
     /* Status bar */
     statusBar = gtk_statusbar_new();
-    gtk_box_pack_start(GTK_BOX(vbox), statusBar, FALSE /*expand*/,
-                       TRUE /*fill*/, 0 /*padding*/);
+    pack_main_window_start(GTK_BOX(vbox), statusBar, FALSE, TRUE, 0);
+
+    /* Add to main window */
     gtk_container_add(GTK_CONTAINER(window), vbox);
 
     /* make sure that everything, window and label, are visible */
     gtk_widget_show_all(window);
 
     /* dont't show the history */
-    gtk_widget_hide(history_tab->tree);
+    gtk_widget_hide(history_vbox);
 
     /* dont't show the contact list */
     gtk_widget_hide(contacts_tab->tree);
 
+    /* show playback scale only if a recorded call is selected */
+    sfl_seekslider_set_display(SFL_SEEKSLIDER(seekslider), SFL_SEEKSLIDER_DISPLAY_PLAY);
+    gtk_widget_set_sensitive(seekslider, FALSE);
+    main_window_hide_playback_scale();
+
     /* don't show waiting layer */
     gtk_widget_hide(waitingLayer);
 
@@ -410,7 +481,7 @@ main_window_zrtp_not_supported(callable_obj_t * c)
 
     if (account != NULL) {
         warning_enabled = account_lookup(account,
-                                         ACCOUNT_ZRTP_NOT_SUPP_WARNING);
+                                         CONFIG_ZRTP_NOT_SUPP_WARNING);
         DEBUG("Warning Enabled %s", warning_enabled);
     } else {
         DEBUG("Account is null callID %s", c->_callID);
@@ -418,7 +489,7 @@ main_window_zrtp_not_supported(callable_obj_t * c)
 
         if (properties)
             warning_enabled = g_hash_table_lookup(properties,
-                                                  ACCOUNT_ZRTP_NOT_SUPP_WARNING);
+                                                  CONFIG_ZRTP_NOT_SUPP_WARNING);
     }
 
     if (utf8_case_equal(warning_enabled, "true")) {
@@ -478,3 +549,47 @@ main_window_confirm_go_clear(callable_obj_t * c)
 
     add_error_dialog(GTK_WIDGET(mini_dialog));
 }
+
+void
+main_window_update_playback_scale(guint current, guint size)
+{
+     sfl_seekslider_update_scale(SFL_SEEKSLIDER(seekslider), current, size);
+}
+
+void
+main_window_set_playback_scale_sensitive()
+{
+    gtk_widget_set_sensitive(seekslider, TRUE);
+}
+
+void
+main_window_set_playback_scale_unsensitive()
+{
+    gtk_widget_set_sensitive(seekslider, FALSE);
+}
+
+void
+main_window_show_playback_scale()
+{
+    gtk_widget_show(seekslider);
+}
+
+void
+main_window_hide_playback_scale()
+{
+    gtk_widget_hide(seekslider);
+    main_window_reset_playback_scale();
+}
+
+void
+main_window_reset_playback_scale()
+{
+    sfl_seekslider_reset((SFLSeekSlider *)seekslider);
+}
+
+
+void
+main_window_pause_keygrabber(gboolean value)
+{
+    pause_grabber = value;
+}
